package org.processmining.models.cnet.importing;

import java.io.InputStream;

import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.framework.abstractplugins.AbstractImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.models.cnet.CausalNet;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserFactory;

/**
 * Imports a CausalNet
 * 
 * @author aadrians
 * @author F. Mannhardt
 * 
 */
@Plugin(name = "Open CNet file", parameterLabels = { "Filename" }, returnLabels = { "Causal net" }, returnTypes = { CausalNet.class }, level = PluginLevel.PeerReviewed)
@UIImportPlugin(description = "Causal Net files", extensions = { "cnet" })
public class ImportCNet extends AbstractImportPlugin {

	protected Object importFromStream(PluginContext context, InputStream input, String filename, long fileSizeInBytes)
			throws Exception {
		context.log("CNet import started.");

		// start parsing
		/*
		 * Get an XML pull parser.
		 */
		XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
		factory.setNamespaceAware(true);
		XmlPullParser xpp = factory.newPullParser();
		/*
		 * Initialize the parser on the provided input.
		 */
		xpp.setInput(input, null);
		/*
		 * Get the first event type.
		 */
		int eventType = xpp.getEventType();
		/*
		 * Skip whatever we find until we've found a start tag.
		 */
		while (eventType != XmlPullParser.START_TAG) {
			eventType = xpp.next();
		}

		CNetImporter importer = new CNetImporter();
		/*
		 * Check whether start tag corresponds to CNet start tag.
		 */
		if (xpp.getName().equals(CNetImporter.STARTTAG)) {
			/*
			 * Yes it does. Import the PNML element.
			 */
			importer.importElement(xpp);
		} else {
			/*
			 * No it does not. Return null to signal failure.
			 */
			context.log("no start tag is found");
			return null;
		}

		context.log("CNet Import finished.");

		// rename results
		CausalNet result = importer.getImportResult();
		context.getFutureResult(0).setLabel(result.getLabel());
		
		return result;
	}

}
