package org.processmining.models.dcnet;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.datapetrinets.expression.GuardExpression;
import org.processmining.models.cnet.CNetBinding;
import org.processmining.models.cnet.CNetNode;
import org.processmining.models.cnet.CausalNet;

import com.google.common.collect.ImmutableSet;

public class DCNet implements DataAwareCausalNet {

	private final CausalNet causalNet;
	private final Map<CNetBinding, GuardExpression> guards = new HashMap<>();
	private final Map<String, Class<?>> variables = new HashMap<>();
	private final Map<CNetNode, Set<String>> writes = new HashMap<>();

	public DCNet(CausalNet causalNet) {
		super();
		this.causalNet = causalNet;
	}

	public CausalNet getCausalNet() {
		return causalNet;
	}

	public Set<CNetBinding> getGuardedBindings() {
		return guards.keySet();
	}

	public GuardExpression getGuardExpression(CNetBinding binding) {
		return guards.get(binding);
	}

	public void setGuardExpression(CNetBinding binding, GuardExpression guardExpression) {
		guards.put(binding, guardExpression);
	}

	public Set<String> getVariables() {
		return variables.keySet();
	}

	public Class<?> getVariableType(String variable) {
		return variables.get(variable);
	}

	public void addVariable(String variable, Class<?> type) {
		variables.put(variable, type);
	}

	public Set<String> getWrites(CNetNode node) {
		Set<String> write = writes.get(node);
		if (write == null) {
			return ImmutableSet.of();
		}
		return write;
	}

	public void addWrite(CNetNode node, String variable) {
		Set<String> existingWrites = writes.get(node);
		if (existingWrites == null) {
			existingWrites = new HashSet<>();
			writes.put(node, existingWrites);
		}
		existingWrites.add(variable);
	}

}