package org.processmining.plugins.causalgraph;

import java.awt.Color;
import java.awt.Component;
import java.util.Comparator;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.models.causalgraph.Relation;
import org.processmining.models.causalgraph.SimpleCausalGraph;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.Ordering;

public class CausalGraphColoredTable extends JTable {

	private static final long serialVersionUID = 1L;

	private static final class LocalRenderer extends DefaultTableCellRenderer {

		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus,
				int row, int col) {
			Component comp = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);

			if (col > 0) {

				double s = Double.valueOf((String) table.getModel().getValueAt(row, col));
				
				Color color = getColorValue(s);				
				comp.setBackground(color);
				comp.setForeground(determineFontColor(Color.BLACK, color));
			} else {
				comp.setBackground(Color.WHITE);
			}

			return (comp);
		}

		private Color getColorValue(double d) {
			/* int color */

			int r = 255, g = 255, b = 255;
			if (d > 0.0) {
				g = (int) ((1.0 - d) * 127.0);
				r = g;
			}
			if (d < 0.0) {
				g = (int) ((d + 1.0) * 127.0);
				b = g;
			}

			return new Color(r, g, b);
		}
	}

	public CausalGraphColoredTable(SimpleCausalGraph causalGraph) {
		super();

		int n = causalGraph.getSetActivities().size();
		String[] columnNames = new String[n + 1];
		String[][] rows = new String[n][n + 1];

		ImmutableList<XEventClass> sortedActivities = Ordering.from(new Comparator<XEventClass>() {

			public int compare(XEventClass o1, XEventClass o2) {
				return o1.getId().compareTo(o2.getId());
			}
		}).immutableSortedCopy(causalGraph.getSetActivities());

		for (int r = 0; r < sortedActivities.size(); r++) {
			XEventClass rowActivity = sortedActivities.get(r);
			rows[r][0] = rowActivity.getId();

			for (int c = 0; c < sortedActivities.size(); c++) {
				XEventClass columnActivity = sortedActivities.get(c);
				if (r == 0) {
					columnNames[c + 1] = columnActivity.getId();
				}

				double v = causalGraph.getCausalRelations()
						.contains(Relation.Factory.create(rowActivity, columnActivity)) ? 1.0 : 0.0;
				;
				rows[r][c + 1] = String.valueOf(v);

			}
		}

		columnNames[0] = "Matrix";

		TableModel model = new DefaultTableModel(rows, columnNames);

		setModel(model);
		setDefaultRenderer(Object.class, new LocalRenderer());
		setOpaque(false);
		setEnabled(false);
	}
	
	private static final Color determineFontColor(final Color textColor, final Color bgColor) {
		if (textColor == null) {
			double val = Math.sqrt(.299 * Math.pow(bgColor.getRed(), 2) + .587 * Math.pow(bgColor.getGreen(), 2)
					+ .114 * Math.pow(bgColor.getBlue(), 2));
			return (val < 130) ? Color.WHITE : Color.BLACK;
		} else {
			return textColor;
		}
	}


}